@@ -110,19 +110,45 @@ NTSTATUS dbwrap_record_delete(struct db_record *rec)
 {
 	NTSTATUS status;
 
-	/*
-	 * Invalidate before rec->delete_rec() is called, give
-	 * rec->delete_rec() the chance to re-validate rec->value.
-	 */
-	rec->value_valid = false;
-
 	status = rec->delete_rec(rec);
 	if (!NT_STATUS_IS_OK(status)) {
 		return status;
 	}
+
+	rec->value = tdb_null;
+
 	return NT_STATUS_OK;
 }
 
+struct dbwrap_merge_dbs_state {
+	struct db_context *to;
+	int flags;
+};
+
+/* Copy a single record to the db_context passed in private_data */
+static int dbwrap_merge_dbs_copy_record(struct db_record *rec,
+					    void *private_data)
+{
+	struct dbwrap_merge_dbs_state *state = private_data;
+
+	TDB_DATA data = dbwrap_record_get_value(rec);
+	TDB_DATA key = dbwrap_record_get_key(rec);
+	NTSTATUS status = dbwrap_store(state->to, key, data, state->flags);
+
+	return NT_STATUS_IS_OK(status) ? 0 : 1;
+}
+
+NTSTATUS
+dbwrap_merge_dbs(struct db_context *to, struct db_context *from, int flags)
+{
+	struct dbwrap_merge_dbs_state state = {.to = to, .flags = flags};
+
+	return dbwrap_traverse(from,
+			       dbwrap_merge_dbs_copy_record,
+			       &state,
+			       NULL);
+}
+
 const char *locked_dbs[DBWRAP_LOCK_ORDER_MAX];
 
 static void debug_lock_order(int level)
@@ -683,22 +709,28 @@ static ssize_t tdb_data_buf(const TDB_DATA *dbufs, int num_dbufs,
 }
 
 
-TDB_DATA dbwrap_merge_dbufs(TALLOC_CTX *mem_ctx,
+NTSTATUS dbwrap_merge_dbufs(TDB_DATA *buf, TALLOC_CTX *mem_ctx,
 			    const TDB_DATA *dbufs, int num_dbufs)
 {
 	ssize_t len = tdb_data_buf(dbufs, num_dbufs, NULL, 0);
-	uint8_t *buf;
 
 	if (len == -1) {
-		return (TDB_DATA) {0};
+		return NT_STATUS_INVALID_PARAMETER;
 	}
 
-	buf = talloc_array(mem_ctx, uint8_t, len);
-	if (buf == NULL) {
-		return (TDB_DATA) {0};
+	if (buf->dsize != len) {
+		uint8_t *tmp;
+
+		tmp = talloc_realloc(mem_ctx, buf->dptr, uint8_t, len);
+		if (tmp == NULL && len != 0) {
+			return NT_STATUS_NO_MEMORY;
+		}
+
+		buf->dptr = tmp;
+		buf->dsize = len;
 	}
 
-	tdb_data_buf(dbufs, num_dbufs, buf, len);
+	tdb_data_buf(dbufs, num_dbufs, buf->dptr, buf->dsize);
 
-	return (TDB_DATA) { .dptr = buf, .dsize = len };
+	return NT_STATUS_OK;
 }
