@@ -25,8 +25,6 @@
 #include "lib/util/dlinklist.h"
 #include "lib/util/charset/charset.h"
 #include "lib/util/charset/charset_proto.h"
-#include "libcli/util/ntstatus.h"
-#include "lib/util/util_str_hex.h"
 
 #ifdef HAVE_ICU_I18N
 #include <unicode/ustring.h>
@@ -215,7 +213,7 @@ static size_t sys_uconv(void *cd,
 
 	/*
 	 * The following transliteration function takes two parameters, the
-	 * lenght of the text to be converted (converted_len) and a limit which
+	 * length of the text to be converted (converted_len) and a limit which
 	 * may be smaller then converted_len. We just set limit to converted_len
 	 * and also ignore the value returned in limit.
 	 */
@@ -274,7 +272,7 @@ static size_t sys_uconv(void *cd,
 #endif
 
 /**
- * This is a simple portable iconv() implementaion.
+ * This is a simple portable iconv() implementation.
  *
  * It only knows about a very small number of character sets - just
  * enough that Samba works on systems that don't have iconv.
@@ -675,8 +673,9 @@ static size_t ucs2hex_pull(void *cd, const char **inbuf, size_t *inbytesleft,
 			 char **outbuf, size_t *outbytesleft)
 {
 	while (*inbytesleft >= 1 && *outbytesleft >= 2) {
-		uint64_t v;
-		NTSTATUS status;
+		uint8_t hi = 0, lo = 0;
+		bool ok;
+
 		if ((*inbuf)[0] != '@') {
 			/* seven bit ascii case */
 			(*outbuf)[0] = (*inbuf)[0];
@@ -692,15 +691,15 @@ static size_t ucs2hex_pull(void *cd, const char **inbuf, size_t *inbytesleft,
 			errno = EINVAL;
 			return -1;
 		}
-		status = read_hex_bytes(&(*inbuf)[1], 4, &v);
 
-		if (!NT_STATUS_IS_OK(status)) {
+		ok = hex_byte(&(*inbuf)[1], &hi) && hex_byte(&(*inbuf)[3], &lo);
+		if (!ok) {
 			errno = EILSEQ;
 			return -1;
 		}
 
-		(*outbuf)[0] = v&0xff;
-		(*outbuf)[1] = v>>8;
+		(*outbuf)[0] = lo;
+		(*outbuf)[1] = hi;
 		(*inbytesleft)  -= 5;
 		(*outbytesleft) -= 2;
 		(*inbuf)  += 5;
@@ -833,6 +832,11 @@ static size_t utf8_pull(void *cd, const char **inbuf, size_t *inbytesleft,
 			}
 			uc[1] = (c[0]>>2) & 0x7;
 			uc[0] = (c[0]<<6) | (c[1]&0x3f);
+			if (uc[1] == 0 && uc[0] < 0x80) {
+				/* this should have been a single byte */
+				errno = EILSEQ;
+				goto error;
+			}
 			c  += 2;
 			in_left  -= 2;
 			out_left -= 2;
@@ -841,14 +845,24 @@ static size_t utf8_pull(void *cd, const char **inbuf, size_t *inbytesleft,
 		}
 
 		if ((c[0] & 0xf0) == 0xe0) {
+			unsigned int codepoint;
 			if (in_left < 3 ||
 			    (c[1] & 0xc0) != 0x80 ||
 			    (c[2] & 0xc0) != 0x80) {
 				errno = EILSEQ;
 				goto error;
 			}
-			uc[1] = ((c[0]&0xF)<<4) | ((c[1]>>2)&0xF);
-			uc[0] = (c[1]<<6) | (c[2]&0x3f);
+			codepoint = ((c[2] & 0x3f)        |
+				     ((c[1] & 0x3f) << 6) |
+				     ((c[0] & 0x0f) << 12));
+
+			if (codepoint < 0x800) {
+				/* this should be a 1 or 2 byte sequence */
+				errno = EILSEQ;
+				goto error;
+			}
+			uc[0] = codepoint & 0xff;
+			uc[1] = codepoint >> 8;
 			c  += 3;
 			in_left  -= 3;
 			out_left -= 2;
@@ -871,15 +885,10 @@ static size_t utf8_pull(void *cd, const char **inbuf, size_t *inbytesleft,
 				((c[1]&0x3f)<<12) |
 				((c[0]&0x7)<<18);
 			if (codepoint < 0x10000) {
-				/* accept UTF-8 characters that are not
-				   minimally packed, but pack the result */
-				uc[0] = (codepoint & 0xFF);
-				uc[1] = (codepoint >> 8);
-				c += 4;
-				in_left -= 4;
-				out_left -= 2;
-				uc += 2;
-				continue;
+				/* reject UTF-8 characters that are not
+				   minimally packed */
+				errno = EILSEQ;
+				goto error;
 			}
 
 			codepoint -= 0x10000;
