@@ -54,11 +54,15 @@
 #include "passdb.h"
 #include "auth.h"
 #include "lib/util/sys_rw.h"
+#include "librpc/rpc/dcerpc_samr.h"
 
 #include "lib/crypto/gnutls_helpers.h"
 #include <gnutls/gnutls.h>
 #include <gnutls/crypto.h>
 
+#undef DBGC_CLASS
+#define DBGC_CLASS DBGC_RPC_SRV
+
 #ifndef ALLOW_CHANGE_PASSWORD
 #if (defined(HAVE_TERMIOS_H) && defined(HAVE_DUP2) && defined(HAVE_SETSID))
 #define ALLOW_CHANGE_PASSWORD 1
@@ -587,7 +591,7 @@ bool chgpasswd(const char *name, const char *rhost, const struct passwd *pass,
 	}
 #endif
 
-	/* A non-PAM password change just doen't make sense without a valid local user */
+	/* A non-PAM password change just doesn't make sense without a valid local user */
 
 	if (pass == NULL) {
 		DEBUG(0, ("chgpasswd: user %s doesn't exist in the UNIX password database.\n", name));
@@ -817,7 +821,7 @@ static NTSTATUS check_oem_password(const char *user,
 				NTSTATUS status = NT_STATUS_ACCESS_DISABLED_BY_POLICY_OTHER;
 				return gnutls_error_to_ntstatus(rc, status);
 			}
-			if (memcmp(verifier, old_nt_hash_encrypted, 16)) {
+			if (!mem_equal_const_time(verifier, old_nt_hash_encrypted, 16)) {
 				DEBUG(0, ("check_oem_password: old nt "
 					  "password doesn't match.\n"));
 				return NT_STATUS_WRONG_PASSWORD;
@@ -848,7 +852,7 @@ static NTSTATUS check_oem_password(const char *user,
 				NTSTATUS status = NT_STATUS_ACCESS_DISABLED_BY_POLICY_OTHER;
 				return gnutls_error_to_ntstatus(rc, status);
 			}
-			if (memcmp(verifier, old_lm_hash_encrypted, 16)) {
+			if (!mem_equal_const_time(verifier, old_lm_hash_encrypted, 16)) {
 				DEBUG(0,("check_oem_password: old lm password doesn't match.\n"));
 				return NT_STATUS_WRONG_PASSWORD;
 			}
@@ -872,7 +876,7 @@ static NTSTATUS check_oem_password(const char *user,
 			NTSTATUS status = NT_STATUS_ACCESS_DISABLED_BY_POLICY_OTHER;
 			return gnutls_error_to_ntstatus(rc, status);
 		}
-		if (memcmp(verifier, old_lm_hash_encrypted, 16)) {
+		if (!mem_equal_const_time(verifier, old_lm_hash_encrypted, 16)) {
 			DEBUG(0,("check_oem_password: old lm password doesn't match.\n"));
 			return NT_STATUS_WRONG_PASSWORD;
 		}
@@ -915,8 +919,8 @@ static bool password_in_history(uint8_t nt_pw[NT_HASH_LEN],
 			 * New format: zero salt and then plain nt hash.
 			 * Directly compare the hashes.
 			 */
-			if (memcmp(nt_pw, old_nt_pw_salted_md5_hash,
-				   SALTED_MD5_HASH_LEN) == 0)
+			if (mem_equal_const_time(nt_pw, old_nt_pw_salted_md5_hash,
+						 SALTED_MD5_HASH_LEN))
 			{
 				return true;
 			}
@@ -945,9 +949,9 @@ static bool password_in_history(uint8_t nt_pw[NT_HASH_LEN],
 			}
 			gnutls_hash_deinit(hash_hnd, new_nt_pw_salted_md5_hash);
 
-			if (memcmp(new_nt_pw_salted_md5_hash,
-				   old_nt_pw_salted_md5_hash,
-				   SALTED_MD5_HASH_LEN) == 0) {
+			if (mem_equal_const_time(new_nt_pw_salted_md5_hash,
+						 old_nt_pw_salted_md5_hash,
+						 SALTED_MD5_HASH_LEN)) {
 				return true;
 			}
 		}
@@ -986,7 +990,7 @@ static bool check_passwd_history(struct samu *sampass, const char *plaintext)
 
 	E_md4hash(plaintext, new_nt_p16);
 
-	if (!memcmp(nt_pw, new_nt_p16, NT_HASH_LEN)) {
+	if (mem_equal_const_time(nt_pw, new_nt_p16, NT_HASH_LEN)) {
 		DEBUG(10,("check_passwd_history: proposed new password for user %s is the same as the current password !\n",
 			pdb_get_username(sampass) ));
 		return True;
@@ -1068,10 +1072,10 @@ NTSTATUS check_password_complexity(const char *username,
  is correct before calling. JRA.
 ************************************************************/
 
-static NTSTATUS change_oem_password(struct samu *hnd, const char *rhost,
-				    char *old_passwd, char *new_passwd,
-				    bool as_root,
-				    enum samPwdChangeReason *samr_reject_reason)
+NTSTATUS change_oem_password(struct samu *hnd, const char *rhost,
+			     char *old_passwd, char *new_passwd,
+			     bool as_root,
+			     enum samPwdChangeReason *samr_reject_reason)
 {
 	uint32_t min_len;
 	uint32_t refuse;
@@ -1234,7 +1238,7 @@ NTSTATUS pass_oem_change(char *user, const char *rhost,
 				       &new_passwd);
 
 	/*
-	 * We must re-load the sam acount information under a mutex
+	 * We must re-load the sam account information under a mutex
 	 * lock to ensure we don't miss any concurrent account lockout
 	 * changes.
 	 */
@@ -1363,3 +1367,53 @@ done:
 
 	return nt_status;
 }
+
+NTSTATUS samr_set_password_aes(TALLOC_CTX *mem_ctx,
+			       const DATA_BLOB *cdk,
+			       struct samr_EncryptedPasswordAES *pwbuf,
+			       char **new_password_str)
+{
+	DATA_BLOB pw_data = data_blob_null;
+	DATA_BLOB new_password = data_blob_null;
+	const DATA_BLOB ciphertext =
+		data_blob_const(pwbuf->cipher, pwbuf->cipher_len);
+	DATA_BLOB iv = data_blob_const(pwbuf->salt, sizeof(pwbuf->salt));
+	NTSTATUS status;
+	bool ok;
+
+	*new_password_str = NULL;
+
+	status = samba_gnutls_aead_aes_256_cbc_hmac_sha512_decrypt(
+		mem_ctx,
+		&ciphertext,
+		cdk,
+		&samr_aes256_enc_key_salt,
+		&samr_aes256_mac_key_salt,
+		&iv,
+		pwbuf->auth_data,
+		&pw_data);
+	if (!NT_STATUS_IS_OK(status)) {
+		return NT_STATUS_WRONG_PASSWORD;
+	}
+
+	ok = decode_pwd_string_from_buffer514(mem_ctx,
+					      pw_data.data,
+					      CH_UTF16,
+					      &new_password);
+	TALLOC_FREE(pw_data.data);
+	if (!ok) {
+		DBG_NOTICE("samr: failed to decode password buffer\n");
+		return NT_STATUS_WRONG_PASSWORD;
+	}
+
+	*new_password_str = talloc_strndup(mem_ctx,
+					   (char *)new_password.data,
+					   new_password.length);
+	TALLOC_FREE(new_password.data);
+	if (*new_password_str == NULL) {
+		return NT_STATUS_NO_MEMORY;
+	}
+	talloc_keep_secret(*new_password_str);
+
+	return NT_STATUS_OK;
+}
