@@ -27,35 +27,11 @@
 #include "../lib/util/util_net.h"
 #include "lib/tls/tls.h"
 
-#if ENABLE_GNUTLS
 #include <gnutls/gnutls.h>
 #include <gnutls/x509.h>
 
 #define DH_BITS 2048
 
-#if defined(HAVE_GNUTLS_DATUM) && !defined(HAVE_GNUTLS_DATUM_T)
-typedef gnutls_datum gnutls_datum_t;
-#endif
-
-/*
- * define our own values in a high range
- */
-#ifndef HAVE_DECL_GNUTLS_CERT_EXPIRED
-#define GNUTLS_CERT_EXPIRED                    0x10000000
-#define REQUIRE_CERT_TIME_CHECKS 1
-#endif
-#ifndef HAVE_DECL_GNUTLS_CERT_NOT_ACTIVATED
-#define GNUTLS_CERT_NOT_ACTIVATED              0x20000000
-#ifndef REQUIRE_CERT_TIME_CHECKS
-#define REQUIRE_CERT_TIME_CHECKS 1
-#endif
-#endif
-#ifndef HAVE_DECL_GNUTLS_CERT_UNEXPECTED_OWNER
-#define GNUTLS_CERT_UNEXPECTED_OWNER           0x40000000
-#endif
-
-#endif /* ENABLE_GNUTLS */
-
 const char *tls_verify_peer_string(enum tls_verify_peer_state verify_peer)
 {
 	switch (verify_peer) {
@@ -84,9 +60,7 @@ struct tstream_tls {
 	struct tstream_context *plain_stream;
 	int error;
 
-#if ENABLE_GNUTLS
-	gnutls_session tls_session;
-#endif /* ENABLE_GNUTLS */
+	gnutls_session_t tls_session;
 
 	enum tls_verify_peer_state verify_peer;
 	const char *peer_name;
@@ -185,12 +159,11 @@ static void tstream_tls_retry_trigger(struct tevent_context *ctx,
 	tstream_tls_retry(stream, true);
 }
 
-#if ENABLE_GNUTLS
 static void tstream_tls_push_trigger_write(struct tevent_context *ev,
 					   struct tevent_immediate *im,
 					   void *private_data);
 
-static ssize_t tstream_tls_push_function(gnutls_transport_ptr ptr,
+static ssize_t tstream_tls_push_function(gnutls_transport_ptr_t ptr,
 					 const void *buf, size_t size)
 {
 	struct tstream_context *stream =
@@ -329,7 +302,7 @@ static void tstream_tls_push_done(struct tevent_req *subreq)
 
 static void tstream_tls_pull_done(struct tevent_req *subreq);
 
-static ssize_t tstream_tls_pull_function(gnutls_transport_ptr ptr,
+static ssize_t tstream_tls_pull_function(gnutls_transport_ptr_t ptr,
 					 void *buf, size_t size)
 {
 	struct tstream_context *stream =
@@ -423,16 +396,14 @@ static void tstream_tls_pull_done(struct tevent_req *subreq)
 
 	tstream_tls_retry(stream, false);
 }
-#endif /* ENABLE_GNUTLS */
 
 static int tstream_tls_destructor(struct tstream_tls *tlss)
 {
-#if ENABLE_GNUTLS
 	if (tlss->tls_session) {
 		gnutls_deinit(tlss->tls_session);
 		tlss->tls_session = NULL;
 	}
-#endif /* ENABLE_GNUTLS */
+
 	return 0;
 }
 
@@ -448,13 +419,9 @@ static ssize_t tstream_tls_pending_bytes(struct tstream_context *stream)
 		return -1;
 	}
 
-#if ENABLE_GNUTLS
 	ret = gnutls_record_check_pending(tlss->tls_session);
 	ret += tlss->read.left;
-#else /* ENABLE_GNUTLS */
-	errno = ENOSYS;
-	ret = -1;
-#endif /* ENABLE_GNUTLS */
+
 	return ret;
 }
 
@@ -564,7 +531,6 @@ static void tstream_tls_retry_read(struct tstream_context *stream)
 		tstream_context_data(stream,
 		struct tstream_tls);
 	struct tevent_req *req = tlss->read.req;
-#if ENABLE_GNUTLS
 	int ret;
 
 	if (tlss->error != 0) {
@@ -599,9 +565,6 @@ static void tstream_tls_retry_read(struct tstream_context *stream)
 
 	tlss->read.left = ret;
 	tstream_tls_readv_crypt_next(req);
-#else /* ENABLE_GNUTLS */
-	tevent_req_error(req, ENOSYS);
-#endif /* ENABLE_GNUTLS */
 }
 
 static int tstream_tls_readv_recv(struct tevent_req *req,
@@ -738,7 +701,6 @@ static void tstream_tls_retry_write(struct tstream_context *stream)
 		tstream_context_data(stream,
 		struct tstream_tls);
 	struct tevent_req *req = tlss->write.req;
-#if ENABLE_GNUTLS
 	int ret;
 
 	if (tlss->error != 0) {
@@ -778,9 +740,6 @@ static void tstream_tls_retry_write(struct tstream_context *stream)
 	}
 
 	tstream_tls_writev_crypt_next(req);
-#else /* ENABLE_GNUTLS */
-	tevent_req_error(req, ENOSYS);
-#endif /* ENABLE_GNUTLS */
 }
 
 static int tstream_tls_writev_recv(struct tevent_req *req,
@@ -848,7 +807,6 @@ static void tstream_tls_retry_disconnect(struct tstream_context *stream)
 		tstream_context_data(stream,
 		struct tstream_tls);
 	struct tevent_req *req = tlss->disconnect.req;
-#if ENABLE_GNUTLS
 	int ret;
 
 	if (tlss->error != 0) {
@@ -878,9 +836,6 @@ static void tstream_tls_retry_disconnect(struct tstream_context *stream)
 	}
 
 	tevent_req_done(req);
-#else /* ENABLE_GNUTLS */
-	tevent_req_error(req, ENOSYS);
-#endif /* ENABLE_GNUTLS */
 }
 
 static int tstream_tls_disconnect_recv(struct tevent_req *req,
@@ -909,20 +864,21 @@ static const struct tstream_context_ops tstream_tls_ops = {
 	.disconnect_recv	= tstream_tls_disconnect_recv,
 };
 
-struct tstream_tls_params {
-#if ENABLE_GNUTLS
-	gnutls_certificate_credentials x509_cred;
-	gnutls_dh_params dh_params;
+struct tstream_tls_params_internal {
+	gnutls_certificate_credentials_t x509_cred;
+	gnutls_dh_params_t dh_params;
 	const char *tls_priority;
-#endif /* ENABLE_GNUTLS */
 	bool tls_enabled;
 	enum tls_verify_peer_state verify_peer;
 	const char *peer_name;
 };
 
-static int tstream_tls_params_destructor(struct tstream_tls_params *tlsp)
+struct tstream_tls_params {
+	struct tstream_tls_params_internal *internal;
+};
+
+static int tstream_tls_params_internal_destructor(struct tstream_tls_params_internal *tlsp)
 {
-#if ENABLE_GNUTLS
 	if (tlsp->x509_cred) {
 		gnutls_certificate_free_credentials(tlsp->x509_cred);
 		tlsp->x509_cred = NULL;
@@ -931,12 +887,14 @@ static int tstream_tls_params_destructor(struct tstream_tls_params *tlsp)
 		gnutls_dh_params_deinit(tlsp->dh_params);
 		tlsp->dh_params = NULL;
 	}
-#endif /* ENABLE_GNUTLS */
+
 	return 0;
 }
 
-bool tstream_tls_params_enabled(struct tstream_tls_params *tlsp)
+bool tstream_tls_params_enabled(struct tstream_tls_params *tls_params)
 {
+	struct tstream_tls_params_internal *tlsp = tls_params->internal;
+
 	return tlsp->tls_enabled;
 }
 
@@ -948,40 +906,42 @@ NTSTATUS tstream_tls_params_client(TALLOC_CTX *mem_ctx,
 				   const char *peer_name,
 				   struct tstream_tls_params **_tlsp)
 {
-#if ENABLE_GNUTLS
-	struct tstream_tls_params *tlsp;
+	struct tstream_tls_params *__tlsp = NULL;
+	struct tstream_tls_params_internal *tlsp = NULL;
 	int ret;
 
-	ret = gnutls_global_init();
-	if (ret != GNUTLS_E_SUCCESS) {
-		DEBUG(0,("TLS %s - %s\n", __location__, gnutls_strerror(ret)));
-		return NT_STATUS_NOT_SUPPORTED;
+	__tlsp = talloc_zero(mem_ctx, struct tstream_tls_params);
+	if (__tlsp == NULL) {
+		return NT_STATUS_NO_MEMORY;
 	}
 
-	tlsp = talloc_zero(mem_ctx, struct tstream_tls_params);
-	NT_STATUS_HAVE_NO_MEMORY(tlsp);
-
-	talloc_set_destructor(tlsp, tstream_tls_params_destructor);
+	tlsp = talloc_zero(__tlsp, struct tstream_tls_params_internal);
+	if (tlsp == NULL) {
+		TALLOC_FREE(__tlsp);
+		return NT_STATUS_NO_MEMORY;
+	}
+	talloc_set_destructor(tlsp, tstream_tls_params_internal_destructor);
+	__tlsp->internal = tlsp;
 
 	tlsp->verify_peer = verify_peer;
 	if (peer_name != NULL) {
 		tlsp->peer_name = talloc_strdup(tlsp, peer_name);
 		if (tlsp->peer_name == NULL) {
-			talloc_free(tlsp);
+			TALLOC_FREE(__tlsp);
 			return NT_STATUS_NO_MEMORY;
 		}
 	} else if (tlsp->verify_peer >= TLS_VERIFY_PEER_CA_AND_NAME) {
 		DEBUG(0,("TLS failed to missing peer_name - "
 			 "with 'tls verify peer = %s'\n",
 			 tls_verify_peer_string(tlsp->verify_peer)));
-		talloc_free(tlsp);
+		TALLOC_FREE(__tlsp);
 		return NT_STATUS_INVALID_PARAMETER_MIX;
 	}
 
 	ret = gnutls_certificate_allocate_credentials(&tlsp->x509_cred);
 	if (ret != GNUTLS_E_SUCCESS) {
 		DEBUG(0,("TLS %s - %s\n", __location__, gnutls_strerror(ret)));
-		talloc_free(tlsp);
+		TALLOC_FREE(__tlsp);
 		return NT_STATUS_NO_MEMORY;
 	}
 
@@ -992,7 +952,7 @@ NTSTATUS tstream_tls_params_client(TALLOC_CTX *mem_ctx,
 		if (ret < 0) {
 			DEBUG(0,("TLS failed to initialise cafile %s - %s\n",
 				 ca_file, gnutls_strerror(ret)));
-			talloc_free(tlsp);
+			TALLOC_FREE(__tlsp);
 			return NT_STATUS_CANT_ACCESS_DOMAIN_INFO;
 		}
 	} else if (tlsp->verify_peer >= TLS_VERIFY_PEER_CA_ONLY) {
@@ -1000,7 +960,7 @@ NTSTATUS tstream_tls_params_client(TALLOC_CTX *mem_ctx,
 			 "with 'tls verify peer = %s'\n",
 			 ca_file,
 			 tls_verify_peer_string(tlsp->verify_peer)));
-		talloc_free(tlsp);
+		TALLOC_FREE(__tlsp);
 		return NT_STATUS_INVALID_PARAMETER_MIX;
 	}
 
@@ -1011,7 +971,7 @@ NTSTATUS tstream_tls_params_client(TALLOC_CTX *mem_ctx,
 		if (ret < 0) {
 			DEBUG(0,("TLS failed to initialise crlfile %s - %s\n",
 				 crl_file, gnutls_strerror(ret)));
-			talloc_free(tlsp);
+			TALLOC_FREE(__tlsp);
 			return NT_STATUS_CANT_ACCESS_DOMAIN_INFO;
 		}
 	} else if (tlsp->verify_peer >= TLS_VERIFY_PEER_AS_STRICT_AS_POSSIBLE) {
@@ -1019,23 +979,20 @@ NTSTATUS tstream_tls_params_client(TALLOC_CTX *mem_ctx,
 			 "with 'tls verify peer = %s'\n",
 			 crl_file,
 			 tls_verify_peer_string(tlsp->verify_peer)));
-		talloc_free(tlsp);
+		TALLOC_FREE(__tlsp);
 		return NT_STATUS_INVALID_PARAMETER_MIX;
 	}
 
 	tlsp->tls_priority = talloc_strdup(tlsp, tls_priority);
 	if (tlsp->tls_priority == NULL) {
-		talloc_free(tlsp);
+		TALLOC_FREE(__tlsp);
 		return NT_STATUS_NO_MEMORY;
 	}
 
 	tlsp->tls_enabled = true;
 
-	*_tlsp = tlsp;
+	*_tlsp = __tlsp;
 	return NT_STATUS_OK;
-#else /* ENABLE_GNUTLS */
-	return NT_STATUS_NOT_IMPLEMENTED;
-#endif /* ENABLE_GNUTLS */
 }
 
 struct tstream_tls_connect_state {
@@ -1045,16 +1002,16 @@ struct tstream_tls_connect_state {
 struct tevent_req *_tstream_tls_connect_send(TALLOC_CTX *mem_ctx,
 					     struct tevent_context *ev,
 					     struct tstream_context *plain_stream,
-					     struct tstream_tls_params *tls_params,
+					     struct tstream_tls_params *_tls_params,
 					     const char *location)
 {
 	struct tevent_req *req;
 	struct tstream_tls_connect_state *state;
 	const char *error_pos;
-#if ENABLE_GNUTLS
 	struct tstream_tls *tlss;
+	struct tstream_tls_params_internal *tls_params = NULL;
 	int ret;
-#endif /* ENABLE_GNUTLS */
+	unsigned int flags = GNUTLS_CLIENT;
 
 	req = tevent_req_create(mem_ctx, &state,
 				struct tstream_tls_connect_state);
@@ -1062,7 +1019,6 @@ struct tevent_req *_tstream_tls_connect_send(TALLOC_CTX *mem_ctx,
 		return NULL;
 	}
 
-#if ENABLE_GNUTLS
 	state->tls_stream = tstream_context_create(state,
 						   &tstream_tls_ops,
 						   &tlss,
@@ -1074,6 +1030,23 @@ struct tevent_req *_tstream_tls_connect_send(TALLOC_CTX *mem_ctx,
 	ZERO_STRUCTP(tlss);
 	talloc_set_destructor(tlss, tstream_tls_destructor);
 
+	/*
+	 * Note we need to make sure x509_cred and dh_params
+	 * from tstream_tls_params_internal stay alive for
+	 * the whole lifetime of this session!
+	 *
+	 * See 'man gnutls_credentials_set' and
+	 * 'man gnutls_certificate_set_dh_params'.
+	 *
+	 * Note: here we use talloc_reference() in a way
+	 *       that does not expose it to the caller.
+	 *
+	 */
+	tls_params = talloc_reference(tlss, _tls_params->internal);
+	if (tevent_req_nomem(tls_params, req)) {
+		return tevent_req_post(req, ev);
+	}
+
 	tlss->plain_stream = plain_stream;
 	tlss->verify_peer = tls_params->verify_peer;
 	if (tls_params->peer_name != NULL) {
@@ -1089,24 +1062,43 @@ struct tevent_req *_tstream_tls_connect_send(TALLOC_CTX *mem_ctx,
 		return tevent_req_post(req, ev);
 	}
 
-	ret = gnutls_init(&tlss->tls_session, GNUTLS_CLIENT);
+#ifdef GNUTLS_NO_TICKETS
+	/*
+	 * tls_tstream can't properly handle 'New Session Ticket' messages
+	 * sent 'after' the client sends the 'Finished' message.
+	 * GNUTLS_NO_TICKETS was introduced in GnuTLS 3.5.6.  This flag is to
+	 * indicate the session Flag session should not use resumption with
+	 * session tickets.
+	 */
+	flags |= GNUTLS_NO_TICKETS;
+#endif
+
+	ret = gnutls_init(&tlss->tls_session, flags);
 	if (ret != GNUTLS_E_SUCCESS) {
 		DEBUG(0,("TLS %s - %s\n", __location__, gnutls_strerror(ret)));
 		tevent_req_error(req, EINVAL);
 		return tevent_req_post(req, ev);
 	}
 
-	ret = gnutls_priority_set_direct(tlss->tls_session,
-					 tls_params->tls_priority,
-					 &error_pos);
+	ret = gnutls_set_default_priority(tlss->tls_session);
 	if (ret != GNUTLS_E_SUCCESS) {
-		DEBUG(0,("TLS %s - %s.  Check 'tls priority' option at '%s'\n",
-			 __location__, gnutls_strerror(ret), error_pos));
+		DBG_ERR("TLS %s - %s. Failed to set default priorities\n",
+			__location__, gnutls_strerror(ret));
 		tevent_req_error(req, EINVAL);
 		return tevent_req_post(req, ev);
 	}
 
-	gnutls_priority_set_direct(tlss->tls_session, "NORMAL:+CTYPE-OPENPGP", NULL);
+	if (strlen(tls_params->tls_priority) > 0) {
+		ret = gnutls_priority_set_direct(tlss->tls_session,
+						 tls_params->tls_priority,
+						 &error_pos);
+		if (ret != GNUTLS_E_SUCCESS) {
+			DEBUG(0,("TLS %s - %s.  Check 'tls priority' option at '%s'\n",
+				 __location__, gnutls_strerror(ret), error_pos));
+			tevent_req_error(req, EINVAL);
+			return tevent_req_post(req, ev);
+		}
+	}
 
 	ret = gnutls_credentials_set(tlss->tls_session,
 				     GNUTLS_CRD_CERTIFICATE,
@@ -1117,14 +1109,12 @@ struct tevent_req *_tstream_tls_connect_send(TALLOC_CTX *mem_ctx,
 		return tevent_req_post(req, ev);
 	}
 
-	gnutls_transport_set_ptr(tlss->tls_session, (gnutls_transport_ptr)state->tls_stream);
+	gnutls_transport_set_ptr(tlss->tls_session,
+				 (gnutls_transport_ptr_t)state->tls_stream);
 	gnutls_transport_set_pull_function(tlss->tls_session,
 					   (gnutls_pull_func)tstream_tls_pull_function);
 	gnutls_transport_set_push_function(tlss->tls_session,
 					   (gnutls_push_func)tstream_tls_push_function);
-#if GNUTLS_VERSION_MAJOR < 3
-	gnutls_transport_set_lowat(tlss->tls_session, 0);
-#endif
 
 	tlss->handshake.req = req;
 	tstream_tls_retry_handshake(state->tls_stream);
@@ -1133,10 +1123,6 @@ struct tevent_req *_tstream_tls_connect_send(TALLOC_CTX *mem_ctx,
 	}
 
 	return req;
-#else /* ENABLE_GNUTLS */
-	tevent_req_error(req, ENOSYS);
-	return tevent_req_post(req, ev);
-#endif /* ENABLE_GNUTLS */
 }
 
 int tstream_tls_connect_recv(struct tevent_req *req,
@@ -1172,31 +1158,44 @@ NTSTATUS tstream_tls_params_server(TALLOC_CTX *mem_ctx,
 				   const char *tls_priority,
 				   struct tstream_tls_params **_tlsp)
 {
-	struct tstream_tls_params *tlsp;
-#if ENABLE_GNUTLS
+	struct tstream_tls_params *__tlsp = NULL;
+	struct tstream_tls_params_internal *tlsp = NULL;
 	int ret;
 	struct stat st;
 
 	if (!enabled || key_file == NULL || *key_file == 0) {
-		tlsp = talloc_zero(mem_ctx, struct tstream_tls_params);
-		NT_STATUS_HAVE_NO_MEMORY(tlsp);
-		talloc_set_destructor(tlsp, tstream_tls_params_destructor);
+		__tlsp = talloc_zero(mem_ctx, struct tstream_tls_params);
+		if (__tlsp == NULL) {
+			return NT_STATUS_NO_MEMORY;
+		}
+
+		tlsp = talloc_zero(__tlsp, struct tstream_tls_params_internal);
+		if (tlsp == NULL) {
+			TALLOC_FREE(__tlsp);
+			return NT_STATUS_NO_MEMORY;
+		}
+
+		talloc_set_destructor(tlsp, tstream_tls_params_internal_destructor);
+		__tlsp->internal = tlsp;
 		tlsp->tls_enabled = false;
 
-		*_tlsp = tlsp;
+		*_tlsp = __tlsp;
 		return NT_STATUS_OK;
 	}
 
-	ret = gnutls_global_init();
-	if (ret != GNUTLS_E_SUCCESS) {
-		DEBUG(0,("TLS %s - %s\n", __location__, gnutls_strerror(ret)));
-		return NT_STATUS_NOT_SUPPORTED;
+	__tlsp = talloc_zero(mem_ctx, struct tstream_tls_params);
+	if (__tlsp == NULL) {
+		return NT_STATUS_NO_MEMORY;
 	}
 
-	tlsp = talloc_zero(mem_ctx, struct tstream_tls_params);
-	NT_STATUS_HAVE_NO_MEMORY(tlsp);
+	tlsp = talloc_zero(__tlsp, struct tstream_tls_params_internal);
+	if (tlsp == NULL) {
+		TALLOC_FREE(__tlsp);
+		return NT_STATUS_NO_MEMORY;
+	}
 
-	talloc_set_destructor(tlsp, tstream_tls_params_destructor);
+	talloc_set_destructor(tlsp, tstream_tls_params_internal_destructor);
+	__tlsp->internal = tlsp;
 
 	if (!file_exist(ca_file)) {
 		tls_cert_generate(tlsp, dns_host_name,
@@ -1214,13 +1213,14 @@ NTSTATUS tstream_tls_params_server(TALLOC_CTX *mem_ctx,
 			  key_file,
 			  (unsigned int)st.st_uid, geteuid(),
 			  (unsigned int)(st.st_mode & 0777), 0600));
+		TALLOC_FREE(__tlsp);
 		return NT_STATUS_CANT_ACCESS_DOMAIN_INFO;
 	}
 
 	ret = gnutls_certificate_allocate_credentials(&tlsp->x509_cred);
 	if (ret != GNUTLS_E_SUCCESS) {
 		DEBUG(0,("TLS %s - %s\n", __location__, gnutls_strerror(ret)));
-		talloc_free(tlsp);
+		TALLOC_FREE(__tlsp);
 		return NT_STATUS_NO_MEMORY;
 	}
 
@@ -1231,7 +1231,7 @@ NTSTATUS tstream_tls_params_server(TALLOC_CTX *mem_ctx,
 		if (ret < 0) {
 			DEBUG(0,("TLS failed to initialise cafile %s - %s\n",
 				 ca_file, gnutls_strerror(ret)));
-			talloc_free(tlsp);
+			TALLOC_FREE(__tlsp);
 			return NT_STATUS_CANT_ACCESS_DOMAIN_INFO;
 		}
 	}
@@ -1243,7 +1243,7 @@ NTSTATUS tstream_tls_params_server(TALLOC_CTX *mem_ctx,
 		if (ret < 0) {
 			DEBUG(0,("TLS failed to initialise crlfile %s - %s\n",
 				 crl_file, gnutls_strerror(ret)));
-			talloc_free(tlsp);
+			TALLOC_FREE(__tlsp);
 			return NT_STATUS_CANT_ACCESS_DOMAIN_INFO;
 		}
 	}
@@ -1254,14 +1254,14 @@ NTSTATUS tstream_tls_params_server(TALLOC_CTX *mem_ctx,
 	if (ret != GNUTLS_E_SUCCESS) {
 		DEBUG(0,("TLS failed to initialise certfile %s and keyfile %s - %s\n",
 			 cert_file, key_file, gnutls_strerror(ret)));
-		talloc_free(tlsp);
+		TALLOC_FREE(__tlsp);
 		return NT_STATUS_CANT_ACCESS_DOMAIN_INFO;
 	}
 
 	ret = gnutls_dh_params_init(&tlsp->dh_params);
 	if (ret != GNUTLS_E_SUCCESS) {
 		DEBUG(0,("TLS %s - %s\n", __location__, gnutls_strerror(ret)));
-		talloc_free(tlsp);
+		TALLOC_FREE(__tlsp);
 		return NT_STATUS_NO_MEMORY;
 	}
 
@@ -1274,7 +1274,7 @@ NTSTATUS tstream_tls_params_server(TALLOC_CTX *mem_ctx,
 		if (!dhparms.data) {
 			DEBUG(0,("TLS failed to read DH Parms from %s - %d:%s\n",
 				 dhp_file, errno, strerror(errno)));
-			talloc_free(tlsp);
+			TALLOC_FREE(__tlsp);
 			return NT_STATUS_CANT_ACCESS_DOMAIN_INFO;
 		}
 		dhparms.size = size;
@@ -1285,7 +1285,7 @@ NTSTATUS tstream_tls_params_server(TALLOC_CTX *mem_ctx,
 		if (ret != GNUTLS_E_SUCCESS) {
 			DEBUG(0,("TLS failed to import pkcs3 %s - %s\n",
 				 dhp_file, gnutls_strerror(ret)));
-			talloc_free(tlsp);
+			TALLOC_FREE(__tlsp);
 			return NT_STATUS_CANT_ACCESS_DOMAIN_INFO;
 		}
 	} else {
@@ -1293,7 +1293,7 @@ NTSTATUS tstream_tls_params_server(TALLOC_CTX *mem_ctx,
 		if (ret != GNUTLS_E_SUCCESS) {
 			DEBUG(0,("TLS failed to generate dh_params - %s\n",
 				 gnutls_strerror(ret)));
-			talloc_free(tlsp);
+			TALLOC_FREE(__tlsp);
 			return NT_STATUS_INTERNAL_ERROR;
 		}
 	}
@@ -1302,20 +1302,13 @@ NTSTATUS tstream_tls_params_server(TALLOC_CTX *mem_ctx,
 
 	tlsp->tls_priority = talloc_strdup(tlsp, tls_priority);
 	if (tlsp->tls_priority == NULL) {
-		talloc_free(tlsp);
+		TALLOC_FREE(__tlsp);
 		return NT_STATUS_NO_MEMORY;
 	}
 
 	tlsp->tls_enabled = true;
 
-#else /* ENABLE_GNUTLS */
-	tlsp = talloc_zero(mem_ctx, struct tstream_tls_params);
-	NT_STATUS_HAVE_NO_MEMORY(tlsp);
-	talloc_set_destructor(tlsp, tstream_tls_params_destructor);
-	tlsp->tls_enabled = false;
-#endif /* ENABLE_GNUTLS */
-
-	*_tlsp = tlsp;
+	*_tlsp = __tlsp;
 	return NT_STATUS_OK;
 }
 
@@ -1326,16 +1319,15 @@ struct tstream_tls_accept_state {
 struct tevent_req *_tstream_tls_accept_send(TALLOC_CTX *mem_ctx,
 					    struct tevent_context *ev,
 					    struct tstream_context *plain_stream,
-					    struct tstream_tls_params *tlsp,
+					    struct tstream_tls_params *_tlsp,
 					    const char *location)
 {
 	struct tevent_req *req;
 	struct tstream_tls_accept_state *state;
 	struct tstream_tls *tlss;
 	const char *error_pos;
-#if ENABLE_GNUTLS
+	struct tstream_tls_params_internal *tlsp = NULL;
 	int ret;
-#endif /* ENABLE_GNUTLS */
 
 	req = tevent_req_create(mem_ctx, &state,
 				struct tstream_tls_accept_state);
@@ -1354,7 +1346,22 @@ struct tevent_req *_tstream_tls_accept_send(TALLOC_CTX *mem_ctx,
 	ZERO_STRUCTP(tlss);
 	talloc_set_destructor(tlss, tstream_tls_destructor);
 
-#if ENABLE_GNUTLS
+	/*
+	 * Note we need to make sure x509_cred and dh_params
+	 * from tstream_tls_params_internal stay alive for
+	 * the whole lifetime of this session!
+	 *
+	 * See 'man gnutls_credentials_set' and
+	 * 'man gnutls_certificate_set_dh_params'.
+	 *
+	 * Note: here we use talloc_reference() in a way
+	 *       that does not expose it to the caller.
+	 */
+	tlsp = talloc_reference(tlss, _tlsp->internal);
+	if (tevent_req_nomem(tlsp, req)) {
+		return tevent_req_post(req, ev);
+	}
+
 	tlss->plain_stream = plain_stream;
 
 	tlss->current_ev = ev;
@@ -1370,16 +1377,26 @@ struct tevent_req *_tstream_tls_accept_send(TALLOC_CTX *mem_ctx,
 		return tevent_req_post(req, ev);
 	}
 
-	ret = gnutls_priority_set_direct(tlss->tls_session,
-					 tlsp->tls_priority,
-					 &error_pos);
+	ret = gnutls_set_default_priority(tlss->tls_session);
 	if (ret != GNUTLS_E_SUCCESS) {
-		DEBUG(0,("TLS %s - %s.  Check 'tls priority' option at '%s'\n",
-			 __location__, gnutls_strerror(ret), error_pos));
+		DBG_ERR("TLS %s - %s. Failed to set default priorities\n",
+			__location__, gnutls_strerror(ret));
 		tevent_req_error(req, EINVAL);
 		return tevent_req_post(req, ev);
 	}
 
+	if (strlen(tlsp->tls_priority) > 0) {
+		ret = gnutls_priority_set_direct(tlss->tls_session,
+						 tlsp->tls_priority,
+						 &error_pos);
+		if (ret != GNUTLS_E_SUCCESS) {
+			DEBUG(0,("TLS %s - %s.  Check 'tls priority' option at '%s'\n",
+				 __location__, gnutls_strerror(ret), error_pos));
+			tevent_req_error(req, EINVAL);
+			return tevent_req_post(req, ev);
+		}
+	}
+
 	ret = gnutls_credentials_set(tlss->tls_session, GNUTLS_CRD_CERTIFICATE,
 				     tlsp->x509_cred);
 	if (ret != GNUTLS_E_SUCCESS) {
@@ -1392,14 +1409,12 @@ struct tevent_req *_tstream_tls_accept_send(TALLOC_CTX *mem_ctx,
 					      GNUTLS_CERT_REQUEST);
 	gnutls_dh_set_prime_bits(tlss->tls_session, DH_BITS);
 
-	gnutls_transport_set_ptr(tlss->tls_session, (gnutls_transport_ptr)state->tls_stream);
+	gnutls_transport_set_ptr(tlss->tls_session,
+				 (gnutls_transport_ptr_t)state->tls_stream);
 	gnutls_transport_set_pull_function(tlss->tls_session,
 					   (gnutls_pull_func)tstream_tls_pull_function);
 	gnutls_transport_set_push_function(tlss->tls_session,
 					   (gnutls_push_func)tstream_tls_push_function);
-#if GNUTLS_VERSION_MAJOR < 3
-	gnutls_transport_set_lowat(tlss->tls_session, 0);
-#endif
 
 	tlss->handshake.req = req;
 	tstream_tls_retry_handshake(state->tls_stream);
@@ -1408,10 +1423,6 @@ struct tevent_req *_tstream_tls_accept_send(TALLOC_CTX *mem_ctx,
 	}
 
 	return req;
-#else /* ENABLE_GNUTLS */
-	tevent_req_error(req, ENOSYS);
-	return tevent_req_post(req, ev);
-#endif /* ENABLE_GNUTLS */
 }
 
 static void tstream_tls_retry_handshake(struct tstream_context *stream)
@@ -1420,7 +1431,6 @@ static void tstream_tls_retry_handshake(struct tstream_context *stream)
 		tstream_context_data(stream,
 		struct tstream_tls);
 	struct tevent_req *req = tlss->handshake.req;
-#if ENABLE_GNUTLS
 	int ret;
 
 	if (tlss->error != 0) {
@@ -1453,9 +1463,6 @@ static void tstream_tls_retry_handshake(struct tstream_context *stream)
 		unsigned int status = UINT32_MAX;
 		bool ip = true;
 		const char *hostname = NULL;
-#ifndef HAVE_GNUTLS_CERTIFICATE_VERIFY_PEERS3
-		bool need_crt_checks = false;
-#endif
 
 		if (tlss->peer_name != NULL) {
 			ip = is_ipaddress(tlss->peer_name);
@@ -1482,7 +1489,6 @@ static void tstream_tls_retry_handshake(struct tstream_context *stream)
 			}
 		}
 
-#ifdef HAVE_GNUTLS_CERTIFICATE_VERIFY_PEERS3
 		ret = gnutls_certificate_verify_peers3(tlss->tls_session,
 						       hostname,
 						       &status);
@@ -1492,98 +1498,6 @@ static void tstream_tls_retry_handshake(struct tstream_context *stream)
 			tevent_req_error(req, tlss->error);
 			return;
 		}
-#else /* not HAVE_GNUTLS_CERTIFICATE_VERIFY_PEERS3 */
-		ret = gnutls_certificate_verify_peers2(tlss->tls_session, &status);
-		if (ret != GNUTLS_E_SUCCESS) {
-			DEBUG(1,("TLS %s - %s\n", __location__, gnutls_strerror(ret)));
-			tlss->error = EIO;
-			tevent_req_error(req, tlss->error);
-			return;
-		}
-
-		if (status == 0) {
-			if (hostname != NULL) {
-				need_crt_checks = true;
-			}
-#ifdef REQUIRE_CERT_TIME_CHECKS
-			need_crt_checks = true;
-#endif
-		}
-
-		if (need_crt_checks) {
-			gnutls_x509_crt crt;
-			const gnutls_datum *cert_list;
-			unsigned int cert_list_size = 0;
-#ifdef REQUIRE_CERT_TIME_CHECKS
-			time_t now = time(NULL);
-			time_t tret = -1;
-#endif
-
-			cert_list = gnutls_certificate_get_peers(tlss->tls_session,
-								 &cert_list_size);
-			if (cert_list == NULL) {
-				cert_list_size = 0;
-			}
-			if (cert_list_size == 0) {
-				DEBUG(1,("TLS %s - cert_list_size == 0\n",
-					 __location__));
-				tlss->error = EIO;
-				tevent_req_error(req, tlss->error);
-				return;
-			}
-
-			ret = gnutls_x509_crt_init(&crt);
-			if (ret != GNUTLS_E_SUCCESS) {
-				DEBUG(1,("TLS %s - %s\n", __location__,
-					 gnutls_strerror(ret)));
-				tlss->error = EIO;
-				tevent_req_error(req, tlss->error);
-				return;
-			}
-			ret = gnutls_x509_crt_import(crt,
-						     &cert_list[0],
-						     GNUTLS_X509_FMT_DER);
-			if (ret != GNUTLS_E_SUCCESS) {
-				DEBUG(1,("TLS %s - %s\n", __location__,
-					 gnutls_strerror(ret)));
-				gnutls_x509_crt_deinit(crt);
-				tlss->error = EIO;
-				tevent_req_error(req, tlss->error);
-				return;
-			}
-
-			if (hostname != NULL) {
-				ret = gnutls_x509_crt_check_hostname(crt,
-								     hostname);
-				if (ret == 0) {
-					status |= GNUTLS_CERT_INVALID;
-					status |= GNUTLS_CERT_UNEXPECTED_OWNER;
-				}
-			}
-
-#ifndef HAVE_DECL_GNUTLS_CERT_NOT_ACTIVATED
-			/*
-			 * GNUTLS_CERT_NOT_ACTIVATED is defined by ourself
-			 */
-			tret = gnutls_x509_crt_get_activation_time(crt);
-			if ((tret == -1) || (now > tret)) {
-				status |= GNUTLS_CERT_INVALID;
-				status |= GNUTLS_CERT_NOT_ACTIVATED;
-			}
-#endif
-#ifndef HAVE_DECL_GNUTLS_CERT_EXPIRED
-			/*
-			 * GNUTLS_CERT_EXPIRED is defined by ourself
-			 */
-			tret = gnutls_certificate_expiration_time_peers(tlss->tls_session);
-			if ((tret == -1) || (now > tret)) {
-				status |= GNUTLS_CERT_INVALID;
-				status |= GNUTLS_CERT_EXPIRED;
-			}
-#endif
-			gnutls_x509_crt_deinit(crt);
-		}
-#endif
 
 		if (status != 0) {
 			DEBUG(1,("TLS %s - check failed for "
@@ -1614,9 +1528,6 @@ static void tstream_tls_retry_handshake(struct tstream_context *stream)
 	}
 
 	tevent_req_done(req);
-#else /* ENABLE_GNUTLS */
-	tevent_req_error(req, ENOSYS);
-#endif /* ENABLE_GNUTLS */
 }
 
 int tstream_tls_accept_recv(struct tevent_req *req,
