@@ -36,6 +36,7 @@
 #include "librpc/gen_ndr/ndr_samr_c.h"
 #include "librpc/gen_ndr/ndr_security.h"
 #include "param/param.h"
+#include "lib/crypto/gnutls_helpers.h"
 
 #define TEST_MACHINE_NAME "samsynctest"
 #define TEST_WKSTA_MACHINE_NAME "samsynctest2"
@@ -61,11 +62,11 @@ static NTSTATUS test_SamLogon(struct torture_context *tctx,
 	union netr_Validation validation;
 	uint8_t authoritative;
 	struct dcerpc_binding_handle *b = p->binding_handle;
+	int rc;
 
 	ninfo.identity_info.domain_name.string = domain;
 	ninfo.identity_info.parameter_control = 0;
-	ninfo.identity_info.logon_id_low = 0;
-	ninfo.identity_info.logon_id_high = 0;
+	ninfo.identity_info.logon_id = 0;
 	ninfo.identity_info.account_name.string = account_name;
 	ninfo.identity_info.workstation.string = workstation;
 	generate_random_buffer(ninfo.challenge,
@@ -73,7 +74,11 @@ static NTSTATUS test_SamLogon(struct torture_context *tctx,
 	if (nt_hash) {
 		ninfo.nt.length = 24;
 		ninfo.nt.data = talloc_array(mem_ctx, uint8_t, 24);
-		SMBOWFencrypt(nt_hash->hash, ninfo.challenge, ninfo.nt.data);
+		rc = SMBOWFencrypt(nt_hash->hash, ninfo.challenge,
+				   ninfo.nt.data);
+		if (rc != 0) {
+			return gnutls_error_to_ntstatus(rc, NT_STATUS_ACCESS_DISABLED_BY_POLICY_OTHER);
+		}
 	} else {
 		ninfo.nt.length = 0;
 		ninfo.nt.data = NULL;
@@ -82,7 +87,11 @@ static NTSTATUS test_SamLogon(struct torture_context *tctx,
 	if (lm_hash) {
 		ninfo.lm.length = 24;
 		ninfo.lm.data = talloc_array(mem_ctx, uint8_t, 24);
-		SMBOWFencrypt(lm_hash->hash, ninfo.challenge, ninfo.lm.data);
+		rc = SMBOWFencrypt(lm_hash->hash, ninfo.challenge,
+				   ninfo.lm.data);
+		if (rc != 0) {
+			return gnutls_error_to_ntstatus(rc, NT_STATUS_ACCESS_DISABLED_BY_POLICY_OTHER);
+		}
 	} else {
 		ninfo.lm.length = 0;
 		ninfo.lm.data = NULL;
@@ -319,8 +328,8 @@ static struct sec_desc_buf *samsync_query_lsa_sec_desc(struct torture_context *t
 		if (!security_descriptor_mask_equal(sd1.sd, sdbuf->sd, \
  			    ~SEC_DESC_SACL_PRESENT)) {\
 			torture_comment(tctx, "Security Descriptor Mismatch for %s:\n", #sd1);\
-		        ndr_print_debug((ndr_print_fn_t)ndr_print_security_descriptor, "SamSync", sd1.sd);\
-		        ndr_print_debug((ndr_print_fn_t)ndr_print_security_descriptor, "SamR", sdbuf->sd);\
+			NDR_PRINT_DEBUG(security_descriptor, sd1.sd);\
+			NDR_PRINT_DEBUG(security_descriptor, sdbuf->sd);\
 			ret = false;\
 		}\
 	}\
@@ -463,7 +472,7 @@ static bool samsync_handle_user(struct torture_context *tctx, TALLOC_CTX *mem_ct
 {
 	uint32_t rid = delta->delta_id_union.rid;
 	struct netr_DELTA_USER *user = delta->delta_union.user;
-	struct netr_SamInfo3 *info3;
+	struct netr_SamInfo3 *info3 = NULL;
 	struct samr_Password lm_hash;
 	struct samr_Password nt_hash;
 	struct samr_Password *lm_hash_p = NULL;
