@@ -20,31 +20,55 @@
 
 #include "includes.h"
 #include "torture/torture.h"
-#include "nsswitch/winbind_client.h"
+#include "nsswitch/libwbclient/wbclient.h"
+#include "nsswitch/winbind_nss_config.h"
+#include "nsswitch/winbind_struct_protocol.h"
+#include "nsswitch/libwbclient/wbclient_internal.h"
 #include "libcli/security/security.h"
 #include "librpc/gen_ndr/netlogon.h"
 #include "param/param.h"
 #include "../libcli/auth/pam_errors.h"
 #include "torture/winbind/proto.h"
+#include "lib/util/string_wrappers.h"
 
 #define DO_STRUCT_REQ_REP_EXT(op,req,rep,expected,strict,warnaction,cmt) do { \
+	const char *__cmt = (cmt); \
+	wbcErr __wbc_status = WBC_ERR_UNKNOWN_FAILURE; \
 	NSS_STATUS __got, __expected = (expected); \
-	__got = winbindd_request_response(NULL, op, req, rep); \
+	__wbc_status = wbcRequestResponse(NULL, op, req, rep); \
+	switch (__wbc_status) { \
+	case WBC_ERR_SUCCESS: \
+		__got = NSS_STATUS_SUCCESS; \
+		break; \
+	case WBC_ERR_WINBIND_NOT_AVAILABLE: \
+		__got = NSS_STATUS_UNAVAIL; \
+		break; \
+	case WBC_ERR_DOMAIN_NOT_FOUND: \
+		__got = NSS_STATUS_NOTFOUND; \
+		break; \
+	default: \
+		torture_result(torture, TORTURE_FAIL, \
+				__location__ ": " __STRING(op) \
+				" returned unmapped %s, expected nss %d%s%s", \
+				wbcErrorString(__wbc_status), __expected, \
+				(__cmt) ? ": " : "", \
+				(__cmt) ? (__cmt) : ""); \
+		return false; \
+	} \
 	if (__got != __expected) { \
-		const char *__cmt = (cmt); \
 		if (strict) { \
 			torture_result(torture, TORTURE_FAIL, \
 				__location__ ": " __STRING(op) \
-				" returned %d, expected %d%s%s", \
-				__got, __expected, \
+				" returned %s, got %d , expected %d%s%s", \
+				wbcErrorString(__wbc_status), __got, __expected, \
 				(__cmt) ? ": " : "", \
 				(__cmt) ? (__cmt) : ""); \
 			return false; \
 		} else { \
 			torture_warning(torture, \
 				__location__ ": " __STRING(op) \
-				" returned %d, expected %d%s%s", \
-				__got, __expected, \
+				" returned %s, got %d , expected %d%s%s", \
+				wbcErrorString(__wbc_status), __got, __expected, \
 				(__cmt) ? ": " : "", \
 				(__cmt) ? (__cmt) : ""); \
 			warnaction; \
@@ -1110,6 +1134,29 @@ static bool torture_winbind_struct_lookup_name_sid(struct torture_context *tortu
 	return true;
 }
 
+static bool torture_winbind_struct_lookup_sids_invalid(
+	struct torture_context *torture)
+{
+	struct winbindd_request req = {0};
+	struct winbindd_response rep = {0};
+	bool strict = torture_setting_bool(torture, "strict mode", false);
+	bool ok;
+
+	torture_comment(torture,
+			"Running WINBINDD_LOOKUP_SIDS (struct based)\n");
+
+	ok = true;
+	DO_STRUCT_REQ_REP_EXT(WINBINDD_LOOKUPSIDS, &req, &rep,
+			      NSS_STATUS_NOTFOUND,
+			      strict,
+			      ok=false,
+			      talloc_asprintf(
+				      torture,
+				      "invalid lookupsids succeeded"));
+
+	return ok;
+}
+
 struct torture_suite *torture_winbind_struct_init(TALLOC_CTX *ctx)
 {
 	struct torture_suite *suite = torture_suite_create(ctx, "struct");
@@ -1132,6 +1179,10 @@ struct torture_suite *torture_winbind_struct_init(TALLOC_CTX *ctx)
 	torture_suite_add_simple_test(suite, "getpwent", torture_winbind_struct_getpwent);
 	torture_suite_add_simple_test(suite, "endpwent", torture_winbind_struct_endpwent);
 	torture_suite_add_simple_test(suite, "lookup_name_sid", torture_winbind_struct_lookup_name_sid);
+	torture_suite_add_simple_test(
+		suite,
+		"lookup_sids_invalid",
+		torture_winbind_struct_lookup_sids_invalid);
 
 	suite->description = talloc_strdup(suite, "WINBIND - struct based protocol tests");
 
